/*
 * Copyright (c) 2001, DecisionSoft Limited All rights reserved.
 * Please see LICENSE.TXT for more information.
 */

#include <iostream>

// DOM includes

#include <xercesc/dom/DOM.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/PlatformUtils.hpp>

// Pathan includes

#include <dom3-xpath/XPathNSResolver.hpp>
#include <dom3-xpath/XPathExpression.hpp>
#include <dom3-xpath/XPathEvaluator.hpp>
#include <DOMutils/XStr.hpp>

// Local include

#include "DOMSerialise.hpp"

using namespace std;

int main(void) {

  // Standard Xerces-C initalisation code

  try {
    XERCES_CPP_NAMESPACE_QUALIFIER XMLPlatformUtils::Initialize();
  }
  catch(const XERCES_CPP_NAMESPACE_QUALIFIER XMLException& toCatch) {
    cerr << "Error during Xerces-c Initialization.\n"
	 << "Exception message:"
	 << XERCES_CPP_NAMESPACE_QUALIFIER XMLString::transcode(toCatch.getMessage()) << endl;
    return 1;
  }

  XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser *xmlParser = new XERCES_CPP_NAMESPACE_QUALIFIER XercesDOMParser();

  //Parse data.xml into a DOM tree

  xmlParser->setDoNamespaces(true);
  xmlParser->parse("data.xml");

  // Retreive the parsed document

  const XERCES_CPP_NAMESPACE_QUALIFIER DOMDocument *document = xmlParser->getDocument();
  XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *documentElement = document->getDocumentElement();

  // Create an XPathEvaluator class

  // This class is used as a factory for creating XPathExpression and
  // XPathNSResolver (it is rarely used for evaluation in the latest
  // spec [31/10/2001], however, and is somewhat misnamed)

  XPathEvaluator *evaluator = XPathEvaluator::createEvaluator();

  // Create an XPathResult

  // This class contains the result of an XPath expression

  XPathResult *result;

  // Create an XPathNSResolver using the XPathEvaluator factory class

  // The XPathNSResolver class binds XML namespace prefixes to the
  // full namespace string

  XPathNSResolver *docNSResolver = evaluator->createNSResolver(documentElement);

  // Create an XPathExpression using the XPathEvaluator factory class

  // Pass as parameters the XPath expression and the appropriate
  // XPathNSResolver

  // The XPath request any childNode nodes off the parentNode who have
  // a subchildNode child

  XPathExpression *parsedExpression;

  try {
    parsedExpression = evaluator->createExpression(X("/child::xl:parentNode/child::xl:childNode[child::xl:subchildNode]"), docNSResolver);
  }
  catch (const XPathException &e) {
    cout << "XPathException: " << e.getString() << endl;
    return 1;
  }

  // The XPath expression is parsed and stored. It may now be applied
  // to any number of Documents to produce output.

  // The parsed expression is evaluated on the document. We request a
  // node set as output. The result is placed in the variable result.

  try {
    result = parsedExpression->evaluate(documentElement, XPathResult::ORDERED_NODE_SNAPSHOT_TYPE, result);
  }
  catch(const XPathException &e) {
    cout << "XPathException: " << e.getString() << endl;
    return 1;
  }

  // Retreive the nodeset result of the XPath expression

  // Query the result as a Snapshot of nodes
  // This snapshot will not change if the document is subsequently changed

  // Loop through and print out the result

  try {

    cout << "Snapshot" << endl << "---" << endl;

    int i;
    for(i=0;i<result->getSnapshotLength();i++) {
      cout << result->snapshotItem(i) << endl;
    }
  }
  catch(const XPathException &e) {
    cout << "XPathException: " << e.getString() << endl;
    return 1;
  }


  // Revaluate as an ITERATOR_TYPE

  try {
    result = parsedExpression->evaluate(documentElement, XPathResult::ORDERED_NODE_ITERATOR_TYPE, result);
  }
  catch(const XPathException &e) {
    cout << "XPathException: " << e.getString() << endl;
    return 1;
  }

  // Query the result for with an iterator
  // This snapshot will not change if the document is subsequently changed

  // Loop through and print out the result

  try {

    cout << "Iterator" << endl << "---" << endl;

    if(result->getInvalidIteratorState()) {
      cout << "Document invalidated." << endl;
      return 1;
    }

    while(XERCES_CPP_NAMESPACE_QUALIFIER DOMNode *it = result->iterateNext()) {
      cout << it << endl;
    }
  }
  catch(const XPathException &e) {
    cout << "XPathException: " << e.getString() << endl;
    return 1;
  }

  return 0;
}

